"""
Entities module for file and archive representations.
"""

from .compressed_archive import (
    ArchiveHandler,
    ArchiveHandlerChain,
    Bzip2Archive,
    Bzip2Handler,
    CompressedArchive,
    GzipArchive,
    GzipHandler,
    RarArchive,
    RarHandler,
    SevenZipArchive,
    SevenZipHandler,
    TarBz2Archive,
    TarBz2Handler,
    TarGzArchive,
    TarGzHandler,
    TarXzArchive,
    TarXzHandler,
    XzArchive,
    XzHandler,
    ZipArchive,
    ZipHandler,
    get_archive_manager,
)
from .file import File

__all__ = [
    "File",
    "CompressedArchive",
    "ZipArchive",
    "Bzip2Archive",
    "GzipArchive",
    "TarGzArchive",
    "TarBz2Archive",
    "TarXzArchive",
    "XzArchive",
    "RarArchive",
    "SevenZipArchive",
    "ArchiveHandler",
    "ZipHandler",
    "Bzip2Handler",
    "GzipHandler",
    "TarGzHandler",
    "TarBz2Handler",
    "TarXzHandler",
    "XzHandler",
    "RarHandler",
    "SevenZipHandler",
    "ArchiveHandlerChain",
    "get_archive_manager",
]
